% RGB image denoising running file for the paper:
% "Nonparametric Bayesian Dictionary Learning for Analysis of Compressive
% or Incomplete Image Measurements," submitted.
% Coded by: Mingyuan Zhou, ECE, Duke University, mz1@ee.duke.edu,
% mingyuan.zhou@duke.edu
% Verison 0: 06/14/2009
% Version 1: 09/20/2009
% Version 2: 10/21/2009
% Version 3: 10/28/2009
% Version 4: 11/03/2009
% Last update: 03/30/2010

clear all
PatchSize = 7; %patch size

sigma = 25; %noise stand deviation

IMname = 'castle'; %kangroo, mushroom, train, horses

IMin0 = imread([IMname,'.png']);
%IMin0 = imresize(IMin0,0.5);

IMin0=im2double(IMin0);
randn('seed',0)
IMin = IMin0 + sigma/255*randn(size(IMin0));
PSNRIn = -10*log10((mean((IMin(:)-IMin0(:)).^2)));
IterPerRound = ones(PatchSize,PatchSize); %Maximum iteration in each round
IterPerRound(end,end) = 150;

%K = 256;
if size(IMin,1)*size(IMin,2)<=300*300
    K = 256;  %dictionary size
else
    K = 512;
end

DispPSNR = true; %Calculate and display the PSNR or not;

ReduceDictSize = false; %Reduce the ditionary size during training if it is TRUE, can be used to reduce computational complexity 
IsSeparateAlpha = false; %use a separate precision for each factor score vector if it is TRUE. 
InitOption = 'SVD'; %Initialization with 'SVD' or 'Rand'
LearningMode = 'online'; %'online' or 'batch'

[ave,Iout,D,S,Z,idex,Pi,NoiseVar,alpha,phi,PSNR,PSNRave] = BPFA_Denoise(IMin, PatchSize, K, DispPSNR, IsSeparateAlpha, InitOption, LearningMode, IterPerRound, ReduceDictSize, IMin0, sigma, IMname);
PSNROut = PSNRave;
figure;
subplot(1,3,1); imshow(IMin0); title('Original clean image');
subplot(1,3,2); imshow(IMin); title((['Noisy image, ',num2str(PSNRIn),'dB']));
subplot(1,3,3); imshow(ave.Iout); title((['Denoised Image, ',num2str(PSNROut),'dB']));
print(gcf,'-dpng',[IMname '_' num2str(sigma) '_results.png'])
imwrite(ave.Iout, [IMname '_' num2str(sigma) '.png']);
figure;
[temp, Pidex] = sort(sum(Z,1),'descend');
Dsort = D(:,Pidex);
I = DispDictionary(Dsort,'rgb');
imwrite(I, [IMname '_' num2str(sigma) '_dict.png']);
title('The dictionary trained on the noisy image');

